
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266HTTPClient.h>
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

#define LED_PIN    D3
#define LED_COUNT 8
#define IN D5 // push button pin

// CHANGE THESE 5 to match your settings:
#define MAC1 "68:C6:3A:E1:4B:70"
#define IP1 181
#define IP2 183
#define SSID "YOUR_WIFI_HERE"
#define PASS "YOUR_WIFI_PASSWORD_HERE"
// ALSO, we assume your network is 192.168.0.X   change below as appropriate


#define OFF 0
#define CALLED 1
#define CALLING 2
#define ACKNOWLEDGED 3

short myip, otherip;
short lampstatus;

int buttonState = 1;          // the current reading from the input pin
int lastButtonState = LOW;  // the previous reading from the input pin
unsigned long debounceDelay = 50;    // the debounce time; increase if the output flickers

unsigned long lastDebounceTime = -1;
ESP8266WebServer server(80);         //start the webserver on port 80 (standard html port)
WiFiClient myclient;
String other = "http://192.168.0.";
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

void colorWipe(uint32_t color, int wait);

void httpGETRequest(const char* serverName) {
  WiFiClient client;
  HTTPClient http;
  http.begin(client, serverName);
  int httpResponseCode = http.GET();
  http.end();
  return;
}

void handleCall() {
  // responds to caller, then sets its color to purple
  server.send(200);
  colorWipe(strip.Color(255,   0,   255), 0); // Purple
  lampstatus = CALLED;
}
void handleResponse() {
  // responds to caller, then sets its color to green
  server.send(200);
  if (lampstatus == CALLING) {
    colorWipe(strip.Color(0,   255,   0), 0); // Green
    lampstatus = ACKNOWLEDGED;
  }
}

void handleReset() {
  // responds to caller, then switches off lights
  server.send(200);
  lampstatus = OFF;
  colorWipe(strip.Color(0, 0, 0), 0);
}

void setup() {
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(200);
  colorWipe(strip.Color(255, 0, 0), 0);

  short wait_from;
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  String mymac = WiFi.macAddress();
  Serial.println(mymac);
  // figure out which lamp you are based on your MAC address. Run on serial monitor to get it first
  if (mymac == MAC1) {
    Serial.println("I am one");
    myip = IP1;
    otherip = IP2;
    other += String(IP2);
  } else {
    Serial.println("I am two");
    myip = IP2;
    otherip = IP1;
    other += String(IP1);
  }
  IPAddress ip(192, 168, 0, myip);
  IPAddress dns(8, 8, 4, 4);
  IPAddress gateway(192, 168, 0, 1);
  IPAddress subnet(255, 255, 255, 0);
  WiFi.config(ip, dns, gateway, subnet);
  WiFi.begin(SSID, PASS);
  bool lit = true;
  while (WiFi.status() != WL_CONNECTED) {
    wait_from = millis();
    while (millis() <= 500 + wait_from) yield();
    lit = !lit;
    if (lit)
      colorWipe(strip.Color(255, 0, 0), 0);
    else
      colorWipe(strip.Color(0, 0, 0), 0);
  }
  colorWipe(strip.Color(0,0,255),0);
  Serial.println("Wifi connected");
  server.on("/call", handleCall);
  server.on("/response", handleResponse);
  server.on("/reset", handleReset);
  server.begin();

  //digitalWrite(IN, LOW);
  pinMode(IN, INPUT_PULLUP);
  //pinMode(OUT, OUTPUT);
  //digitalWrite(OUT, HIGH);
  //down = false;
  delay(1000);
  colorWipe(strip.Color(0,0,0),0);
  Serial.println("Setup complete");
}


void loop() {
  delay(10);
  server.handleClient();

  int reading = digitalRead(IN);
  // check to see if you just pressed the button
  // (i.e. the input went from LOW to HIGH), and you've waited long enough
  // since the last press to ignore any noise:
  // If the switch , due to noise or pressing:
  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }
  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:
    // if the button state has changed:
    //Serial.println("changed");
    if (reading != buttonState) {
      Serial.println("and now change status");
      buttonState = reading;
      if (buttonState == HIGH) {
        // if button is pressed
        switch (lampstatus) {
          case OFF:
            // if we are off, we are calling
            colorWipe(strip.Color(255, 255, 0), 0);
            lampstatus = CALLING;
            httpGETRequest((other + "/call").c_str());
            break;
          case ACKNOWLEDGED:
          case CALLING:
            // if we are calling or have received an answer, we reset
            lampstatus = OFF;
            colorWipe(strip.Color(0, 0, 0), 0);
            httpGETRequest((other + "/reset").c_str());
            break;
          case CALLED:
            // if we are called, we send response
            httpGETRequest((other + "/response").c_str());
            break;

        }

      }
    }
  }
  lastButtonState = reading;
}

void colorWipe(uint32_t color, int wait) {
  for (int i = 0; i < strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}
